# SPDX-FileCopyrightText: 2017 Tony DiCola, Carter Nelson for Adafruit Industries
# SPDX-License-Identifier: MIT

# MPR121 piano demo.
# Listens to the first 7 inputs of the MPR121 and plays a middle scale note
# when an input is touched.  Note only one note is played at a time!
# For use with microcontrollers or computers with PWM support only!


import board
import busio
import pwmio

# Import MPR121 module.
import adafruit_mpr121

# Configure PWM buzzer and other state:
BUZZER_PIN = board.D9
TONE_ON_DUTY = 2**15  # Duty cycle of tone when turned on, a square wave.
TONE_OFF_DUTY = 0  # Duty cycle of tone when turned off, 0 or no signal.
NOTE_FREQS = [
    261,  # Input 0 = 261 hz = middle C
    294,  # Input 1 = middle D
    329,  # Input 2 = middle E
    349,  # Input 3 = middle F
    392,  # Input 4 = middle G
    440,  # Input 5 = middle A
    493,  # Input 6 = middle B
    0,  # Input 7 = nothing (set to a frequency in hertz!)
    0,  # Input 8
    0,  # Input 9
    0,  # Input 10
    0,
]  # Input 11


# Create I2C bus.
i2c = busio.I2C(board.SCL, board.SDA)

# Create MPR121 class.
mpr121 = adafruit_mpr121.MPR121(i2c)
# Note you can optionally change the address of the device:
# mpr121 = adafruit_mpr121.MPR121(i2c, address=0x91)

# Setup buzzer PWM output.
buzzer = pwmio.PWMOut(BUZZER_PIN, duty_cycle=TONE_OFF_DUTY, frequency=440, variable_frequency=True)

last_note = None
while True:
    # Get touched state for all pins
    touched = mpr121.touched_pins
    # If no pins are touched, be quiet
    if True not in touched:
        last_note = None
        buzzer.duty_cycle = TONE_OFF_DUTY
        continue
    # Get index of touched pin
    note = touched.index(True)
    # Play note if pin is different and has a defined note
    if note != last_note and NOTE_FREQS[note] != 0:
        last_note = note
        buzzer.frequency = NOTE_FREQS[note]
        buzzer.duty_cycle = TONE_ON_DUTY



# SPDX-FileCopyrightText: 2017 Tony DiCola for Adafruit Industries
# SPDX-License-Identifier: MIT

# Simple test of the MPR121 capacitive touch sensor library.
# Will print out a message when any of the 12 capacitive touch inputs of the
# board are touched.  Open the serial REPL after running to see the output.
# Author: Tony DiCola
import time

import board
import busio

# Import MPR121 module.
import adafruit_mpr121

# Create I2C bus.
i2c = busio.I2C(board.SCL, board.SDA)

# Create MPR121 object.
mpr121 = adafruit_mpr121.MPR121(i2c)

# Note you can optionally change the address of the device:
# mpr121 = adafruit_mpr121.MPR121(i2c, address=0x91)

# Loop forever testing each input and printing when they're touched.
while True:
    # Loop through all 12 inputs (0-11).
    for i in range(12):
        # Call is_touched and pass it then number of the input.  If it's touched
        # it will return True, otherwise it will return False.
        if mpr121[i].value:
            print(f"Input {i} touched!")
    time.sleep(0.25)  # Small delay to keep from spamming output messages.



# SPDX-FileCopyrightText: 2019 Tony DiCola, Brennen Bearnes for Adafruit Industries
# SPDX-License-Identifier: MIT

#!/usr/bin/env python3

# Adafruit Raspberry Pi MPR121 Keyboard Example
#
# Allows you to turn touches detected by the MPR121 into key presses on a
# Raspberry Pi.
#
# Dependencies
# ============
#
# Make sure you have the required dependencies by executing the following commands:
#
#   sudo apt-get update
#   sudo apt-get install build-essential python-dev python-pip libudev-dev
#   sudo pip3 install python-uinput
#   sudo pip3 install adafruit-circuitpython-mpr121
#
# Usage
# =====
#
# To use this program you first need to connect the MPR121 board to the Raspberry
# Pi (either connect the HAT directly to the Pi, or wire the I2C pins SCL, SDA to
# the Pi SCL, SDA, VIN to Pi 3.3V, GND to Pi GND).
#
# Next define the mapping of capacitive touch input presses to keyboard
# button presses.  Scroll down to the KEY_MAPPING dictionary definition below
# and adjust the configuration as described in its comments.
#
# Finally run the script as root:
#
#   sudo python3 pi_keyboard.py
#
# Try pressing buttons and you should see key presses made on the Pi!  (Note
# that you need to be logged directly into the Pi to see the keypresses -
# over an SSH or console cable connection, you won't see anything.)
#
# Press Ctrl-C to quit at any time.
#
# Copyright (c) 2014-2019 Adafruit Industries
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import logging
import subprocess
import time

import board
import busio
import uinput

import adafruit_mpr121

# Define mapping of capacitive touch pin presses to keyboard button presses.

# Each line here should define a dict entry that maps the capacitive touch
# input number to an appropriate key press.
#
# For reference the list of possible uinput.KEY_* values you can specify is
# defined in linux/input.h:
# http://www.cs.fsu.edu/~baker/devices/lxr/http/source/linux/include/linux/input.h?v=2.6.11.8
#
# Make sure a cap touch input is defined only once or else the program will
# fail to run!

KEY_MAPPING = {
    0: uinput.KEY_UP,
    1: uinput.KEY_DOWN,
    2: uinput.KEY_LEFT,
    3: uinput.KEY_RIGHT,
    4: uinput.KEY_B,
    5: uinput.KEY_A,
    6: uinput.KEY_ENTER,
    7: uinput.KEY_SPACE,
}

# Sleep this long between polling for events:
EVENT_WAIT_SLEEP_SECONDS = 0.25

# Uncomment to enable debug message logging (might slow down key detection).
# logging.basicConfig(level=logging.DEBUG)

# Make sure uinput kernel module is loaded.
subprocess.check_call(["modprobe", "uinput"])

# Configure virtual keyboard.
device = uinput.Device(KEY_MAPPING.values())

# Create I2C bus.
i2c = busio.I2C(board.SCL, board.SDA)

# Create MPR121 object.
mpr121 = adafruit_mpr121.MPR121(i2c)

# Event loop to wait for pin changes and respond to them.
print("Press Ctrl-C to quit.")
while True:
    # Loop through all defined inputs:
    for pin, key in KEY_MAPPING.items():
        # Call is_touched and pass it then number of the input.  If it's touched
        # it will return True, otherwise it will return False.
        if mpr121[pin].value:
            logging.debug("Input %i touched!", pin)
            logging.debug("Key: %s", key)
            device.emit_click(key)
    time.sleep(EVENT_WAIT_SLEEP_SECONDS)  # Small delay to keep from spamming output messages.



# SPDX-FileCopyrightText: 2017 Tony DiCola for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_mpr121`
====================================================

CircuitPython driver for the MPR121 capacitive touch breakout board.

See usage in the examples/simpletest.py file.

* Author(s): Tony DiCola

Implementation Notes
--------------------

**Hardware:**

* Adafruit `12-Key Capacitive Touch Sensor Breakout - MPR121
  <https://www.adafruit.com/product/1982>`_ (Product ID: 1982)

* Adafruit `12 x Capacitive Touch Shield for Arduino - MPR121
  <https://www.adafruit.com/product/2024>`_ (Product ID: 2024)

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the ESP8622 and M0-based boards:
  https://github.com/adafruit/circuitpython/releases
* Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
"""

import time

try:
    from typing import List, Optional, Tuple

    import busio
except ImportError:
    # typing hint modules not needed or not available in CircuitPython
    pass


from adafruit_bus_device import i2c_device
from micropython import const

__version__ = "2.1.27"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_MPR121.git"

# Register addresses.  Unused registers commented out to save memory.
MPR121_I2CADDR_DEFAULT = const(0x5A)
MPR121_TOUCHSTATUS_L = const(0x00)
# MPR121_TOUCHSTATUS_H   = const(0x01)
MPR121_FILTDATA_0L = const(0x04)
# MPR121_FILTDATA_0H     = const(0x05)
MPR121_BASELINE_0 = const(0x1E)
MPR121_MHDR = const(0x2B)
MPR121_NHDR = const(0x2C)
MPR121_NCLR = const(0x2D)
MPR121_FDLR = const(0x2E)
MPR121_MHDF = const(0x2F)
MPR121_NHDF = const(0x30)
MPR121_NCLF = const(0x31)
MPR121_FDLF = const(0x32)
MPR121_NHDT = const(0x33)
MPR121_NCLT = const(0x34)
MPR121_FDLT = const(0x35)
MPR121_TOUCHTH_0 = const(0x41)
MPR121_RELEASETH_0 = const(0x42)
MPR121_DEBOUNCE = const(0x5B)
MPR121_CONFIG1 = const(0x5C)
MPR121_CONFIG2 = const(0x5D)
# MPR121_CHARGECURR_0    = const(0x5F)
# MPR121_CHARGETIME_1    = const(0x6C)
MPR121_ECR = const(0x5E)
# MPR121_AUTOCONFIG0     = const(0x7B)
# MPR121_AUTOCONFIG1     = const(0x7C)
# MPR121_UPLIMIT         = const(0x7D)
# MPR121_LOWLIMIT        = const(0x7E)
# MPR121_TARGETLIMIT     = const(0x7F)
# MPR121_GPIODIR         = const(0x76)
# MPR121_GPIOEN          = const(0x77)
# MPR121_GPIOSET         = const(0x78)
# MPR121_GPIOCLR         = const(0x79)
# MPR121_GPIOTOGGLE      = const(0x7A)
MPR121_SOFTRESET = const(0x80)


class MPR121_Channel:
    """Represents a single channel on the touch sensor.

    Not meant to be used directly.
    """

    _mpr121: "MPR121"
    _channel: int

    def __init__(self, mpr121: "MPR121", channel: int) -> None:
        """Creates a new ``MPR121_Channel`` instance.

        :param mpr121: An instance of the touch sensor driver.
        :param channel: The channel this instance represents (0-11).
        """
        self._mpr121 = mpr121
        self._channel = channel

    @property
    def value(self) -> bool:
        """Get whether the touch pad is being touched or not."""
        return self._mpr121.touched() & (1 << self._channel) != 0

    @property
    def raw_value(self) -> int:
        """Get the raw touch measurement."""
        return self._mpr121.filtered_data(self._channel)

    @property
    def threshold(self) -> int:
        """Get or set the touch threshold."""
        buf = bytearray(1)
        self._mpr121._read_register_bytes(MPR121_TOUCHTH_0 + 2 * self._channel, buf, 1)
        return buf[0]

    @threshold.setter
    def threshold(self, new_thresh: int) -> None:
        self._mpr121._write_register_byte(MPR121_TOUCHTH_0 + 2 * self._channel, new_thresh)

    @property
    def release_threshold(self) -> int:
        """Get or set the release threshold."""
        buf = bytearray(1)
        self._mpr121._read_register_bytes(MPR121_RELEASETH_0 + 2 * self._channel, buf, 1)
        return buf[0]

    @release_threshold.setter
    def release_threshold(self, new_thresh: int) -> None:
        self._mpr121._write_register_byte(MPR121_RELEASETH_0 + 2 * self._channel, new_thresh)


class MPR121:
    """Driver for the MPR121 capacitive touch breakout board."""

    _i2c: i2c_device.I2CDevice
    _buffer: bytearray
    _channels: List[Optional[MPR121_Channel]]

    def __init__(self, i2c: busio.I2C, address: int = MPR121_I2CADDR_DEFAULT) -> None:
        """Creates a new ``MPR121`` instance.

        :param i2c: An I2C driver.
        :type i2c: class:`busio.I2C`
        :param address: The address of the touch sensor (0x5A - 0x5D).
        :type address: int
        """
        self._i2c = i2c_device.I2CDevice(i2c, address)
        self._buffer = bytearray(2)
        self._channels = [None] * 12
        self.reset()

    def __getitem__(self, key: int) -> MPR121_Channel:
        if key < 0 or key > 11:
            raise IndexError("pin must be a value 0-11")
        if self._channels[key] is None:
            self._channels[key] = MPR121_Channel(self, key)
        return self._channels[key]

    @property
    def touched_pins(self) -> Tuple[bool]:
        """Get a tuple of the touched state for all pins."""
        touched = self.touched()
        return tuple(bool(touched >> i & 1) for i in range(12))

    def _write_register_byte(self, register: int, value: int) -> None:
        # Write a byte value to the specifier register address.
        # MPR121 must be put in Stop Mode to write to most registers
        stop_required = True
        if register == MPR121_ECR or 0x73 <= register <= 0x7A:
            stop_required = False
        with self._i2c:
            if stop_required:
                self._i2c.write(bytes([MPR121_ECR, 0x00]))
            self._i2c.write(bytes([register, value]))
            if stop_required:
                self._i2c.write(bytes([MPR121_ECR, 0x8F]))

    def _read_register_bytes(
        self, register: int, result: bytearray, length: Optional[int] = None
    ) -> None:
        # Read the specified register address and fill the specified result byte
        # array with result bytes.  Make sure result buffer is the desired size
        # of data to read.
        if length is None:
            length = len(result)
        with self._i2c:
            self._i2c.write_then_readinto(bytes([register]), result, in_end=length)

    def reset(self) -> None:
        """Reset the MPR121 into a default state.

        All configurations and states previously set are lost.

        :raises RuntimeError: The sensor is in an invalid config state.
        """
        # Write to the reset register.
        self._write_register_byte(MPR121_SOFTRESET, 0x63)
        time.sleep(0.001)  # This 1ms delay here probably isn't necessary but can't hurt.

        # Set electrode configuration to default values.
        self._write_register_byte(MPR121_ECR, 0x00)

        # Check CDT, SFI, ESI configuration is at default values.
        self._read_register_bytes(MPR121_CONFIG2, self._buffer, 1)
        if self._buffer[0] != 0x24:
            raise RuntimeError("Failed to find MPR121 in expected config state!")

        # Default touch and release thresholds
        for i in range(12):
            self._write_register_byte(MPR121_TOUCHTH_0 + 2 * i, 12)
            self._write_register_byte(MPR121_RELEASETH_0 + 2 * i, 6)

        # Configure baseline filtering control registers.
        self._write_register_byte(MPR121_MHDR, 0x01)
        self._write_register_byte(MPR121_NHDR, 0x01)
        self._write_register_byte(MPR121_NCLR, 0x0E)
        self._write_register_byte(MPR121_FDLR, 0x00)
        self._write_register_byte(MPR121_MHDF, 0x01)
        self._write_register_byte(MPR121_NHDF, 0x05)
        self._write_register_byte(MPR121_NCLF, 0x01)
        self._write_register_byte(MPR121_FDLF, 0x00)
        self._write_register_byte(MPR121_NHDT, 0x00)
        self._write_register_byte(MPR121_NCLT, 0x00)
        self._write_register_byte(MPR121_FDLT, 0x00)

        # Set other configuration registers.
        self._write_register_byte(MPR121_DEBOUNCE, 0)
        self._write_register_byte(MPR121_CONFIG1, 0x10)  # default, 16uA charge current
        self._write_register_byte(MPR121_CONFIG2, 0x20)  # 0.5uS encoding, 1ms period

        # Enable all electrodes.
        self._write_register_byte(MPR121_ECR, 0x8F)  # start with first 5 bits of baseline tracking

    def filtered_data(self, pin: int) -> int:
        """Get the filtered data register value.

        :param pin: The pin to read (0 - 11).
        :type pin: int

        :raises ValueError: Argument ``pin`` is invalid.

        :return: The filtered data value stored in the register.
        :rtype: int
        """
        if pin < 0 or pin > 11:
            raise ValueError("Pin must be a value 0-11.")
        self._read_register_bytes(MPR121_FILTDATA_0L + pin * 2, self._buffer)
        return ((self._buffer[1] << 8) | (self._buffer[0])) & 0xFFFF

    def baseline_data(self, pin: int) -> int:
        """Get the baseline data register value.

        :param pin: The pin to read (0 - 11).
        :type pin: int

        :raises ValueError: Argument ``pin`` is invalid.

        :return: The baseline data value stored in the register.
        :rtype: int
        """
        if pin < 0 or pin > 11:
            raise ValueError("Pin must be a value 0-11.")
        self._read_register_bytes(MPR121_BASELINE_0 + pin, self._buffer, 1)
        return self._buffer[0] << 2

    def touched(self) -> int:
        """Get the touch state of all pins as a 12-bit value.

        :return: A 12-bit value representing the touch state of each
            pin. Each state in the value is represented by either a 1 or
            0; touched or not.
        :rtype: int
        """
        self._read_register_bytes(MPR121_TOUCHSTATUS_L, self._buffer)
        return ((self._buffer[1] << 8) | (self._buffer[0])) & 0xFFFF

    def is_touched(self, pin: int) -> bool:
        """Get if ``pin`` is being touched.

        :raises ValueError: Argument ``pin`` is invalid.

        :return: True if ``pin`` is being touched; otherwise False.
        :rtype: bool
        """
        if pin < 0 or pin > 11:
            raise ValueError("Pin must be a value 0-11.")
        touches = self.touched()
        return (touches & (1 << pin)) > 0
